@@ -1,5 +1,7 @@
 /*
 ** Vsound - a virtual audio loopback cable used for recording /dev/dsp streams
+** Copyright (C) 2004 Nathan Chantrell <nsc@zorg.org>
+** Copyright (C) 2003 Richard Taylor <r.taylor@bcs.org.uk>
 ** Copyright (C) 2000,2001 Erik de Castro Lopo <erikd@zip.com.au>
 ** Copyright (C) 1999 James Henstridge <james@daa.com.au>
 **
@@ -31,6 +33,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <signal.h>
 
 #ifdef ENABLE_DEBUG
 	#include <errno.h>
@@ -125,6 +128,9 @@ typedef int request_t;
 #define DOTSND_MARKER	(MAKE_MARKER ('.', 's', 'n', 'd')) 
 #define DNSDOT_MARKER	(MAKE_MARKER ('d', 'n', 's', '.')) 
 
+
+
+
 /*---------------------------------------------------------------------------- 
 ** Typedefs and enums. 
 */
@@ -155,6 +161,8 @@ static void fix_header (AU_HEADER *au_header) ;
 static void endswap_short_array	(short* buffer, int count) ;
 static int  au_bytes_per_sample (AU_HEADER *au_header) ;
 static u_long usec_diff_timeval (struct timeval *start, struct timeval *end) ;
+static void start_autostop_timer(void);
+static void stop_autostop_timer(void);
 
 /*---------------------------------------------------------------------------- 
 ** Static Data. 
@@ -163,6 +171,7 @@ static u_long usec_diff_timeval (struct timeval *start, struct timeval *end) ;
 static int filefd = -1, dspfd = -1 ;
 static int enable_dspout = 0, enable_stdout = 0, enable_timing = 0 ;
 static int devdsp_format = 0, done_header = 0 ;
+static int stopdelay = 0, ignore_autostop = 1;
 
 static char *datafile = NULL ;
 
@@ -193,6 +202,7 @@ int open (const char *pathname, int flags, ...)
 	va_list args;
 	mode_t mode;
 	int fd ;
+	
 	  
 	if (!func_open)
 		func_open = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open") ;
@@ -205,10 +215,13 @@ int open (const char *pathname, int flags, ...)
 
 	if (strcmp (pathname, "/dev/dsp"))
 	{	fd = func_open (pathname, flags, mode) ;
-		/* DPRINTF ("Not /dev/dsp. open (%s) = %d\n", pathname, fd) ; */
+	        /*DPRINTF ("Not /dev/dsp. open (%s) = %d\n", pathname, fd) ;*/
 		return fd ;
 		} ;
 
+
+	stop_autostop_timer();
+
 	/* Opening /dev/dsp device so reset done_header. */
 	done_header = 0 ;
 	au_header.datasize = 0xFFFFFFFF ;
@@ -237,7 +250,7 @@ int	ioctl (int fd, request_t request, ...)
 {	static int (*func_ioctl) (int, request_t, void *) = NULL;
 	va_list args;
 	void *argp;
-	
+
 	if (! func_ioctl)
 		func_ioctl = (int (*) (int, request_t, void *)) dlsym (REAL_LIBC, "ioctl") ;
 	va_start (args, request) ;
@@ -272,7 +285,8 @@ int	ioctl (int fd, request_t request, ...)
 int close (int fd)
 {	static int (*func_close) (int) = NULL;
 	int retval = 0 ;
-
+	
+	
 	DPRINTF ("close (%d)\n", fd) ;
 
 	if (! func_close)
@@ -292,8 +306,10 @@ int close (int fd)
 		/* Closing /dev/dsp so reset done_header. */
 		done_header = 0 ;
 		au_header.datasize = 0xFFFFFFFF ;
+		start_autostop_timer();
 		} ;
 
+
 	return retval ;
 }	/* close */
 
@@ -301,6 +317,7 @@ ssize_t write (int fd, const void *buf, size_t count)
 {	static ssize_t (*func_write) (int, const void*, size_t) = NULL ;
 	ssize_t  retval = 0 ;
 
+
 	if (! func_write)
 		func_write = (ssize_t (*) (int, const void*, size_t)) dlsym (REAL_LIBC, "write") ;
 		
@@ -358,6 +375,7 @@ ssize_t write (int fd, const void *buf, size_t count)
 			usleep (usec_sleep) ;
 		} ;
 
+
 	return retval ;
 } 	/* write */
 
@@ -422,6 +440,13 @@ void dsp_init (void)
 		}
 	else
 		DPRINTF ("Output goes to file.\n") ;
+
+	cptr = getenv("VSOUND_STOPDELAY");
+	if (cptr)
+	  stopdelay = atoi(cptr);
+	  DPRINTF("Autostoping after %d seconds of inactivity\n", stopdelay);
+	  
+	  
 		
 } /* dsp_init */
 
@@ -607,3 +632,66 @@ u_long usec_diff_timeval (struct timeval *start, struct timeval *end)
 	return diff ;
 } /* usec_diff_timeval */
 
+
+/* Signal handler for SIGQUIT. */
+void SIGALRM_handler (int signum)
+{
+  DPRINTF ("Autostop alarm\n") ;
+  if (!ignore_autostop){
+    DPRINTF ("Sending sigint\n") ;   
+    exit(1);
+  }
+}
+
+static 
+void start_autostop_timer(void)
+{
+  static int firsttime = 1;
+  static struct itimerval timerval;
+  struct itimerval otimerval;
+  struct sigaction sa;
+
+  if (firsttime && stopdelay) {
+    sigemptyset (&sa.sa_mask);
+    sa.sa_flags = 0;
+    
+    /* Register the handler for SIGALARM. */
+    sa.sa_handler = SIGALRM_handler;
+    sigaction (SIGALRM, &sa, 0);
+          
+    timerval.it_interval.tv_sec = (long)stopdelay;
+    timerval.it_interval.tv_usec = 0;
+    timerval.it_value.tv_sec = (long)stopdelay;
+    timerval.it_value.tv_usec = 0;
+  }
+    
+  if (stopdelay)
+    {     ignore_autostop = 0;
+          setitimer (ITIMER_REAL, &timerval, &otimerval);
+          DPRINTF ("Watchdog started from -> %ld \n", otimerval.it_value.tv_sec) ;
+    }
+
+} /* start_autostop_timer */
+
+static
+void stop_autostop_timer(void)
+{
+  static int firsttime = 1;
+  static struct itimerval timerval;
+  struct itimerval otimerval;
+
+
+  if (firsttime && stopdelay) {
+    timerval.it_interval.tv_sec = (long)0;
+    timerval.it_interval.tv_usec = 0;
+    timerval.it_value.tv_sec = (long)0;
+    timerval.it_value.tv_usec = 0;
+  }
+
+  if (stopdelay) {
+    ignore_autostop = 1;
+    setitimer (ITIMER_REAL, &timerval, &otimerval);
+    DPRINTF ("Watchdog stopped at -> %ld \n", otimerval.it_value.tv_sec) ;
+  }
+    
+} /* stop_autostop_timer */
